"""add new fields to textbook, defaults to others.

Revision ID: a6446496220c
Revises: f936aadf86f1
Create Date: 2025-03-31 15:57:11.465771

"""

from typing import Sequence, Union

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "a6446496220c"
down_revision: Union[str, None] = "f936aadf86f1"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("activities", schema=None) as batch_op:
        batch_op.alter_column("description", existing_type=sa.VARCHAR(), nullable=True)
        batch_op.alter_column("sources", existing_type=sa.VARCHAR(), nullable=True)
        batch_op.alter_column("authors", existing_type=sa.VARCHAR(), nullable=True)

    with op.batch_alter_table("textbooks", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "status",
                sa.Enum("draft", "published", name="textbookstatus"),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("edition", sa.String(), nullable=False))
        batch_op.add_column(sa.Column("schema_version", sa.String(), nullable=False))
        batch_op.add_column(
            sa.Column(
                "attributes",
                sa.String(),
                nullable=False,
            )
        )
        batch_op.alter_column("authors", existing_type=sa.VARCHAR(), nullable=True)
        batch_op.alter_column("reviewers", existing_type=sa.VARCHAR(), nullable=True)

    with op.batch_alter_table("topics", schema=None) as batch_op:
        batch_op.alter_column("sources", existing_type=sa.VARCHAR(), nullable=True)
        batch_op.alter_column("authors", existing_type=sa.VARCHAR(), nullable=True)

    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("topics", schema=None) as batch_op:
        batch_op.alter_column("authors", existing_type=sa.VARCHAR(), nullable=False)
        batch_op.alter_column("sources", existing_type=sa.VARCHAR(), nullable=False)

    with op.batch_alter_table("textbooks", schema=None) as batch_op:
        batch_op.alter_column("reviewers", existing_type=sa.VARCHAR(), nullable=False)
        batch_op.alter_column("authors", existing_type=sa.VARCHAR(), nullable=False)
        batch_op.drop_column("attributes")
        batch_op.drop_column("schema_version")
        batch_op.drop_column("edition")
        batch_op.drop_column("status")

    with op.batch_alter_table("activities", schema=None) as batch_op:
        batch_op.alter_column("authors", existing_type=sa.VARCHAR(), nullable=False)
        batch_op.alter_column("sources", existing_type=sa.VARCHAR(), nullable=False)
        batch_op.alter_column("description", existing_type=sa.VARCHAR(), nullable=False)

    # ### end Alembic commands ###
